@@ -134,6 +134,8 @@ class Fixations(object):
         if attributes is not None:
             self.__attributes__ = list(self.__attributes__)
             for name, value in attributes.items():
+                if key not in self.__attributes__:
+                    self.__attributes__.append(key)
                 setattr(self, name, value)
 
     @classmethod
@@ -186,6 +188,9 @@ class Fixations(object):
     def __getitem__(self, indices):
         return self.filter(indices)
 
+    def __len__(self):
+        return len(self.x)
+
     def filter(self, inds):
         """
         Create new fixations object which contains only the fixations with indexes in inds
