@@ -97,7 +97,7 @@ def get_toronto(location=None):
     @param location: If and where to cache the dataset. The dataset
                      will be stored in the subdirectory `toronto` of
                      location and read from there, if already present.
-    @return: Stimuli, FixationTrains
+    @return: Stimuli, Fixations
 
     .. warning::
         At the moment, the subjects stated in the FixationTrains object
@@ -135,19 +135,17 @@ def get_toronto(location=None):
             stimuli = create_stimuli(stimuli_src_location, stimuli_filenames, stimuli_target_location)
 
             points = loadmat(os.path.join(temp_dir, 'eyetrackingdata', 'fixdens', 'origfixdata.mat'))['white']
-            ts = []
             xs = []
             ys = []
             ns = []
             subjects = []
             for n in range(len(stimuli.stimuli)):
                 _ys, _xs = np.nonzero(points[0, n])
-                xs.extend([[x] for x in _xs])
-                ys.extend([[y] for y in _ys])
+                xs.extend(_xs)
+                ys.extend(_ys)
                 ns.extend([n for x in _xs])
-                ts.extend([[0] for x in _xs])
                 subjects.extend([0 for x in _xs])
-            fixations = FixationTrains.from_fixation_trains(xs, ys, ts, ns, subjects)
+            fixations = Fixations.create_without_history(xs, ys, ns, subjects=subjects)
         if location:
             stimuli.to_hdf5(os.path.join(location, 'stimuli.hdf5'))
             fixations.to_hdf5(os.path.join(location, 'fixations.hdf5'))
@@ -259,7 +257,7 @@ def get_toronto_with_subjects(location=None):
     return stimuli, fixations
 
 
-def _get_mit1003(dataset_name, location=None, include_initial_fixation=False, only_1024_by_768=False):
+def _get_mit1003(dataset_name, location=None, include_initial_fixation=False, only_1024_by_768=False, replace_initial_invalid_fixations=False):
     """
     .. seealso::
 
@@ -369,6 +367,8 @@ def _get_mit1003(dataset_name, location=None, include_initial_fixation=False, on
             train_durations = []
             for n, stimulus in enumerate(stimuli_filenames):
                 stimulus_size = stimuli.sizes[n]
+                height, width = stimulus_size
+
                 for subject_id, subject in enumerate(subjects):
                     subject_name = os.path.split(subject)[-1]
                     outfile = '{0}_{1}.mat'.format(stimulus, subject_name)
@@ -376,36 +376,91 @@ def _get_mit1003(dataset_name, location=None, include_initial_fixation=False, on
                     fix_data = mat_data['fixations']
                     starts = mat_data['starts']
                     _durations = mat_data['durations']
-                    x = []
-                    y = []
-                    t = []
-                    duration = []
+
+                    _xs = mat_data['fixations'][:, 0]
+                    _ys = mat_data['fixations'][:, 1]
+                    _ts = mat_data['starts'].flatten()
+                    _durations = mat_data['durations'].flatten()
+                    full_durations = _durations.copy()
+
+                    # this would be consistent with how it was handled in matlab
+                    # however, originally I ported this slightly differnt
+                    # and now I'm staying consistent withing pysaliency
+                    # valid_indices = (
+                    #    (np.floor(_xs) > 0) & (np.floor(_xs) < width)
+                    #    & (np.floor(_ys) > 0) & (np.floor(_ys) < height))
+                    valid_indices = (
+                        (_xs > 0) & (_xs < width)
+                        & (_ys > 0) & (_ys < height))
+
+                    _xs = _xs[valid_indices]
+                    _ys = _ys[valid_indices]
+                    _ts = _ts[valid_indices]
+                    _durations = _durations[valid_indices]
+
+                    #_xs = _xs - 1  # matlab uses one-based indexing
+                    #_ys = _ys - 1  # matlab uses one-based indexing
+                    #_ts = _ts - 1  # data starts with t == 1, we want to use 0
+                    _ts = _ts / 240.0  # Eye Tracker rate = 240Hz
+                    _durations = _durations / 1000  # data is in ms, we want seconds
+
+                    if not valid_indices[0] and (replace_initial_invalid_fixations or first_fixation > 0):
+                        # if first fixation is invalid, no valid initial fixation is removed in the dataset
+                        # for those cases, we add a central fixation with same duration as the invalid fixation
+                        _xs = np.hstack(([0.5 * width], _xs))
+                        _ys = np.hstack(([0.5 * height], _ys))
+                        _ts = np.hstack(([0], _ts))
+                        _durations = np.hstack(([full_durations[0] / 1000], _durations))
+
                     # if first_fixation == 1 the first fixation is skipped, as done
                     # by Judd.
-                    # TODO: This contains a subtle bug: if the first fixation is invalid,
-                    # the next fixation is not fixed. Therefore, the dataset still
-                    # contains a few initial fixations.
-                    for i in range(first_fixation, fix_data.shape[0]):
-                        if fix_data[i, 0] < 0 or fix_data[i, 1] < 0:
-                            continue
-                        if fix_data[i, 0] >= stimulus_size[1] or fix_data[i, 1] >= stimulus_size[0]:
-                            continue
-                        x.append(fix_data[i, 0])
-                        y.append(fix_data[i, 1])
-                        t.append(starts[0, i] / 240.0)  # Eye Tracker rate = 240Hz
-                        duration_hist.append(np.array(duration))
-                        duration.append(_durations[0, i] / 1000)  # data is in ms, we want seconds
-                    xs.append(x)
-                    ys.append(y)
-                    ts.append(t)
+                    _xs = _xs[first_fixation:]
+                    _ys = _ys[first_fixation:]
+                    _ts = _ts[first_fixation:]
+                    _durations = _durations[first_fixation:]
+
+                    xs.append(_xs)
+                    ys.append(_ys)
+                    ts.append(_ts)
                     ns.append(n)
                     train_subjects.append(subject_id)
-                    train_durations.append(duration)
+                    train_durations.append(_durations)
+
+                    for i in range(len(_durations)):
+                        duration_hist.append(_durations[:i])
+
+                    # x = []
+                    # y = []
+                    # t = []
+                    # duration = []
+                    # # TODO: This contains a subtle inconsistency: if the first fixation is invalid,
+                    # # the next fixation is _not_ skipped. Therefore, the dataset still
+                    # # contains a few "real" initial fixations. However, this is consistent
+                    # # with how the dataset has been used in the past, so we are
+                    # # keeping it.
+                    # for i in range(first_fixation, fix_data.shape[0]):
+                    #     if fix_data[i, 0] < 0 or fix_data[i, 1] < 0:
+                    #         continue
+                    #     if fix_data[i, 0] >= stimulus_size[1] or fix_data[i, 1] >= stimulus_size[0]:
+                    #         continue
+                    #     x.append(fix_data[i, 0])
+                    #     y.append(fix_data[i, 1])
+                    #     t.append(starts[0, i] / 240.0)  # Eye Tracker rate = 240Hz
+                    #     duration_hist.append(np.array(duration))
+                    #     duration.append(_durations[0, i] / 1000)  # data is in ms, we want seconds
+                    # xs.append(x)
+                    # ys.append(y)
+                    # ts.append(t)
+                    # ns.append(n)
+                    # train_subjects.append(subject_id)
+                    # train_durations.append(duration)
 
             attributes = {
+                # duration_hist contains for each fixation the durations of the previous fixations in the scanpath
                 'duration_hist': build_padded_2d_array(duration_hist),
             }
             scanpath_attributes = {
+                # train_durations contains the fixation durations for each scanpath
                 'train_durations': build_padded_2d_array(train_durations),
             }
             fixations = FixationTrains.from_fixation_trains(xs, ys, ts, ns, train_subjects, attributes=attributes, scanpath_attributes=scanpath_attributes)
@@ -640,16 +695,54 @@ def get_cat2000_test(location=None):
     return stimuli
 
 
-def get_cat2000_train(location=None, include_initial_fixation=True):
+def get_cat2000_train(location=None, version='1'):
+    """
+    Loads or downloads and caches the CAT2000 training dataset. The dataset
+    consists of 2000 images of
+    sizes: 1080 x 1920.
+    and the fixations of 18 subjects per image under
+    free viewing conditions with 5 seconds presentation time.
+
+    All fixations outside of the image are discarded. This includes
+    blinks.
+
+    in v1.1 a few details have been fixed that change the dataset:
+    - the quite common repeated fixations are deduplicated (happens serveral thousand times)
+    - a values of 1 is substracted from the x and y values to account for the one-based indexing in MATLAB
+
+    To use v1.1, call this function with version='1.1'
+
+    @type  location: string, defaults to `None`
+    @param location: If and where to cache the dataset. The dataset
+                     will be stored in the subdirectory `toronto` of
+                     location and read from there, if already present.
+    @return: Stimuli, FixationTrains
+
+    .. note::
+        This code needs a working matlab or octave installation.
+
+    .. seealso::
+
+        Ali Borji, Laurent Itti. CAT2000: A Large Scale Fixation Dataset for Boosting Saliency Research [CVPR 2015 workshop on "Future of Datasets"]
+
+        http://saliency.mit.edu/datasets.html
+    """
+
     name = 'CAT2000_train'
 
-    if not include_initial_fixation:
-        name += '_without_initial_fixation'
+    if version == '1':
+        warnings.warn("You are still using version 1.0 of the CAT2000 dataset implementation in pysaliency. Please upgrade to version 1.1 with pysaliency.get_cat2000_train(..., version='1.1'). This will become the default in pysaliency version 1.0. For more details please see the docstring of this function")
+        get_fn = _get_cat2000_train
+    elif version == '1.1':
+        get_fn = _get_cat2000_train_v1_1
+        name += '_v1.1'
+    else:
+        raise ValueError(version)
 
-    return _get_cat2000_train(name=name, location=location, include_initial_fixation=include_initial_fixation)
+    return get_fn(name=name, location=location)
 
 
-def _get_cat2000_train(name, location, include_initial_fixation):
+def _get_cat2000_train(name, location):
     """
     Loads or downloads and caches the CAT2000 training dataset. The dataset
     consists of 2000 images of
@@ -675,10 +768,7 @@ def _get_cat2000_train(name, location, include_initial_fixation):
 
         http://saliency.mit.edu/datasets.html
     """
-    if include_initial_fixation:
-        first_fixation = 0
-    else:
-        first_fixation = 1
+    first_fixation = 0
 
     if location:
         location = os.path.join(location, name)
@@ -774,6 +864,139 @@ def _get_cat2000_train(name, location, include_initial_fixation):
     return stimuli, fixations
 
 
+def _get_cat2000_train_v1_1(name, location):
+    """
+    Loads or downloads and caches the CAT2000 training dataset. The dataset
+    consists of 2000 images of
+    sizes: 1080 x 1920.
+    and the fixations of 18 subjects per image under
+    free viewing conditions with 5 seconds presentation time.
+
+    All fixations outside of the image are discarded. This includes
+    blinks.
+
+    in v1_1 a few details have been fixed:
+    - the quite common repeated fixations are deduplicated (happens serveral thousand times)
+    - a values of 1 is substracted from the x and y values to account for the one-based indexing in MATLAB
+
+    @type  location: string, defaults to `None`
+    @param location: If and where to cache the dataset. The dataset
+                     will be stored in the subdirectory `toronto` of
+                     location and read from there, if already present.
+    @return: Stimuli, FixationTrains
+
+    .. note::
+        This code needs a working matlab or octave installation.
+
+    .. seealso::
+
+        Ali Borji, Laurent Itti. CAT2000: A Large Scale Fixation Dataset for Boosting Saliency Research [CVPR 2015 workshop on "Future of Datasets"]
+
+        http://saliency.mit.edu/datasets.html
+    """
+    first_fixation = 0
+
+    if location:
+        location = os.path.join(location, name)
+        if os.path.exists(location):
+            stimuli = _load(os.path.join(location, 'stimuli.hdf5'))
+            fixations = _load(os.path.join(location, 'fixations.hdf5'))
+            return stimuli, fixations
+        os.makedirs(location)
+    with atomic_directory_setup(location):
+        with TemporaryDirectory(cleanup=True) as temp_dir:
+            download_and_check('http://saliency.mit.edu/trainSet.zip',
+                               os.path.join(temp_dir, 'trainSet.zip'),
+                               '56ad5c77e6c8f72ed9ef2901628d6e48')
+
+            # Stimuli
+            print('Creating stimuli')
+            f = zipfile.ZipFile(os.path.join(temp_dir, 'trainSet.zip'))
+            f.extractall(temp_dir)
+
+            stimuli_src_location = os.path.join(temp_dir, 'trainSet', 'Stimuli')
+            stimuli_target_location = os.path.join(location, 'Stimuli') if location else None
+            images = glob.glob(os.path.join(stimuli_src_location, '**', '*.jpg'))
+            images = [os.path.relpath(img, start=stimuli_src_location) for img in images]
+            stimuli_filenames = natsorted(images)
+            stimulus_category_names = [os.path.dirname(filename) for filename in stimuli_filenames]
+            category_names = sorted(set(stimulus_category_names))
+            categories = [category_names.index(category_name) for category_name in stimulus_category_names]
+            attributes = {'category': categories}
+
+            stimuli = create_stimuli(stimuli_src_location, stimuli_filenames, stimuli_target_location, attributes=attributes)
+
+            # FixationTrains
+
+            print('Creating fixations')
+
+            with open(os.path.join(temp_dir, 'load_cat2000.m'), 'wb') as f:
+                f.write(resource_string(__name__, 'scripts/{}'.format('load_cat2000.m')))
+
+            # It is vital _not_ to store the extracted fixations in the main
+            # directory where matlab is running, as matlab will check the timestamp
+            # of all files in this directory very often. This leads to heavy
+            # performance penalties and would make matlab run for more than an
+            # hour.
+            out_path = 'extracted'
+            os.makedirs(os.path.join(temp_dir, out_path))
+            run_matlab_cmd('load_cat2000;', cwd=temp_dir)
+
+            print('Extracting fixations. This can take some minutes.')
+            print('Warning: In the IPython Notebook, the output is shown on the console instead of the notebook.')
+
+            xs = []
+            ys = []
+            ts = []
+            ns = []
+            train_subjects = []
+            subject_dict = {}
+
+            files = natsorted(glob.glob(os.path.join(temp_dir, out_path, '*.mat')))
+            for f in progressinfo(files):
+                mat_data = loadmat(f)
+                fix_data = mat_data['data']
+                name = mat_data['name'][0]
+                n = int(os.path.basename(f).split('fix', 1)[1].split('_')[0]) - 1
+                stimulus_size = stimuli.sizes[n]
+                _, _, subject = name.split('.eye')[0].split('-')
+                if subject not in subject_dict:
+                    subject_dict[subject] = len(subject_dict)
+                subject_id = subject_dict[subject]
+
+                x = []
+                y = []
+                t = []
+                for i in range(first_fixation, fix_data.shape[0]):
+                    new_x = fix_data[i, 0] - 1  # one-based indexing in MATLAB
+                    new_y = fix_data[i, 1] - 1  # one-based indexing in MATLAB
+                    if new_x < 0 or new_y < 0:
+                        continue
+                    if new_x >= stimulus_size[1] or new_y >= stimulus_size[0]:
+                        continue
+                    if any(np.isnan(fix_data[i])):  # skip invalid data
+                        continue
+
+                    if x and x[-1] == new_x and y[-1] == new_y:
+                        # repeated fixation, skip
+                        continue
+
+                    x.append(new_x)
+                    y.append(new_y)
+                    t.append(len(x))  # Eye Tracker rate = 240Hz
+                xs.append(x)
+                ys.append(y)
+                ts.append(t)
+                ns.append(n)
+                train_subjects.append(subject_id)
+            fixations = FixationTrains.from_fixation_trains(xs, ys, ts, ns, train_subjects)
+
+        if location:
+            stimuli.to_hdf5(os.path.join(location, 'stimuli.hdf5'))
+            fixations.to_hdf5(os.path.join(location, 'fixations.hdf5'))
+    return stimuli, fixations
+
+
 def get_iSUN(location=None):
     """
     Loads or downloads and caches the iSUN dataset.
@@ -1122,10 +1345,10 @@ def _get_SALICON_fixations(location, name, edition='2015', fixation_type='mouse'
             fixations_file = os.path.join(temp_dir, 'fixations.zip')
 
             if edition == '2015':
-                download_file_from_google_drive('0B2hsWbciDVedWHFiMUVVWFRZTE0', fixations_file)
+                download_file_from_google_drive('1WVEiXba-I4GN33f0uUl4KhaN1rK7qzs6', fixations_file)
                 check_file_hash(fixations_file, '9a22db9d718200fb90252e5010c004c4')
             elif edition == '2017':
-                download_file_from_google_drive('0B2hsWbciDVedS1lBZHprdXFoZkU', fixations_file)
+                download_file_from_google_drive('1P-jeZXCsjoKO79OhFUgnj6FGcyvmLDPj', fixations_file)
                 check_file_hash(fixations_file, '462b70f4f9e8ea446ac628e46cea8d3d')
 
             f = zipfile.ZipFile(fixations_file)
@@ -1206,7 +1429,7 @@ def get_SALICON_val(edition='2015', fixation_type='mouse', location=None):
             return stimuli, fixations
     stimuli_train, stimuli_val, stimuli_test, fixations_train, fixations_val = get_SALICON(location=location, edition=edition, fixation_type=fixation_type)
 
-    return stimuli_val, stimuli_val
+    return stimuli_val, fixations_val
 
 
 def get_SALICON_test(edition='2015', fixation_type='mouse', location=None):
@@ -1682,3 +1905,188 @@ def get_NUSEF_public(location=None):
             stimuli.to_hdf5(os.path.join(location, 'stimuli.hdf5'))
             fixations.to_hdf5(os.path.join(location, 'fixations.hdf5'))
     return stimuli, fixations
+
+
+def get_PASCAL_S(location=None):
+    """
+    Loads or downloads and caches the PASCAL-S dataset.
+    The dataset consists of 850 images from the PASCAL-VOC
+    2010 validation set with fixation from 12 subjects
+    during 2s free-viewing.
+
+    Note that here only the eye movement data from PASCAL-S
+    is included. The original dataset also provides
+    salient object segmentation data.
+
+    @type  location: string, defaults to `None`
+    @param location: If and where to cache the dataset. The dataset
+                     will be stored in the subdirectory `PASCAL-S` of
+                     location and read from there, if already present.
+    @return: Stimuli, FixationTrains
+
+    .. seealso::
+
+        Yin Li, Xiaodi Hu, Christof Koch, James M. Rehg, Alan L. Yuille:
+        The Secrets of Salient Object Segmentation. CVPR 2014.
+
+        http://cbs.ic.gatech.edu/salobj/
+    """
+    if location:
+        location = os.path.join(location, 'PASCAL-S')
+        if os.path.exists(location):
+            stimuli = _load(os.path.join(location, 'stimuli.hdf5'))
+            fixations = _load(os.path.join(location, 'fixations.hdf5'))
+            return stimuli, fixations
+        os.makedirs(location)
+
+    n_stimuli = 850
+
+    with atomic_directory_setup(location):
+        with TemporaryDirectory(cleanup=True) as temp_dir:
+
+            download_and_check('http://cbs.ic.gatech.edu/salobj/download/salObj.zip',
+                               os.path.join(temp_dir, 'salObj.zip'),
+                               'e48b4e5deac08bddcaec55ce56e4d420')
+
+            # Stimuli
+            print('Creating stimuli')
+            f = zipfile.ZipFile(os.path.join(temp_dir, 'salObj.zip'))
+            f.extractall(temp_dir)
+
+            stimuli_src_location = os.path.join(temp_dir, 'datasets', 'imgs', 'pascal')
+            stimuli_filenames = ['{}.jpg'.format(i + 1) for i in range(n_stimuli)]
+
+            stimuli_target_location = os.path.join(location, 'Stimuli') if location else None
+            stimuli = create_stimuli(stimuli_src_location, stimuli_filenames, stimuli_target_location)
+
+            print('Creating fixations')
+
+            train_xs = []
+            train_ys = []
+            train_ts = []
+            train_ns = []
+            train_subjects = []
+
+            import h5py  # we don't import globally to avoid depending on h5py
+            with h5py.File(os.path.join(temp_dir, 'datasets', 'fixations', 'pascalFix.mat'), mode='r') as hdf5_file:
+                fixation_data = [hdf5_file[hdf5_file['fixCell'][0, stimulus_index]][:] for stimulus_index in range(n_stimuli)]
+
+            for n in range(n_stimuli):
+                ys, xs, subject_ids = fixation_data[n]
+                for subject in sorted(set(subject_ids)):
+                    subject_inds = subject_ids == subject
+                    if not np.any(subject_inds):
+                        continue
+
+                    train_xs.append(xs[subject_inds] - 1)  # data is 1-indexed in matlab
+                    train_ys.append(ys[subject_inds] - 1)
+                    train_ts.append(np.arange(subject_inds.sum()))
+                    train_ns.append(n)
+                    train_subjects.append(subject - 1)  # subjects are 1-indexed in matlab
+
+        fixations = FixationTrains.from_fixation_trains(train_xs, train_ys, train_ts, train_ns, train_subjects)
+
+        if location:
+            stimuli.to_hdf5(os.path.join(location, 'stimuli.hdf5'))
+            fixations.to_hdf5(os.path.join(location, 'fixations.hdf5'))
+    return stimuli, fixations
+
+
+def get_DUT_OMRON(location=None):
+    """
+    Loads or downloads the DUT-OMRON fixation dataset.
+    The dataset consists of 5168 natural images with
+    a maximal size of 400 pixel and eye movement data
+    from 5 subjects in a 2 second free viewing task.
+    The eye movement data has been filtered and preprocessed,
+    see the dataset documentation for more details.
+
+    Note that the dataset contains subject ids but they
+    might not be consisten across images.
+
+    @type  location: string, defaults to `None`
+    @param location: If and where to cache the dataset. The dataset
+                     will be stored in the subdirectory `DUT-OMRON` of
+                     location and read from there, if already present.
+    @return: Stimuli, FixationTrains
+
+    .. seealso::
+
+        Chuan Yang, Lihe Zhang, Huchuan Lu, Xiang Ruan, Minghsuan Yang. Saliency Detection Via Graph-Based Manifold Ranking, CVPR2013.
+
+        http://saliencydetection.net/dut-omron
+    """
+    if location:
+        location = os.path.join(location, 'DUT-OMRON')
+        if os.path.exists(location):
+            stimuli = _load(os.path.join(location, 'stimuli.hdf5'))
+            fixations = _load(os.path.join(location, 'fixations.hdf5'))
+            return stimuli, fixations
+        os.makedirs(location)
+
+    n_fixations = 0
+
+    with atomic_directory_setup(location):
+        with TemporaryDirectory(cleanup=True) as temp_dir:
+
+            download_and_check('http://saliencydetection.net/dut-omron/download/DUT-OMRON-image.zip',
+                               os.path.join(temp_dir, 'DUT-OMRON-image.zip'),
+                               'a8951db9297afacf78bc0e5079103cf1')
+
+            download_and_check('http://saliencydetection.net/dut-omron/download/DUT-OMRON-eye-fixations.zip',
+                               os.path.join(temp_dir, 'DUT-OMRON-eye-fixations.zip'),
+                               'd9f4f83fcc78b1e5efb579ae9fb0edc2')
+
+            # Stimuli
+            print('Creating stimuli')
+            f = zipfile.ZipFile(os.path.join(temp_dir, 'DUT-OMRON-image.zip'))
+            f.extractall(temp_dir)
+
+            stimuli_src_location = os.path.join(temp_dir, 'DUT-OMRON-image')
+            images = glob.glob(os.path.join(stimuli_src_location, '*.jpg'))
+            images = [os.path.relpath(img, start=stimuli_src_location) for img in images]
+            stimuli_filenames = sorted(images)
+
+            stimuli_target_location = os.path.join(location, 'Stimuli') if location else None
+            stimuli = create_stimuli(stimuli_src_location, stimuli_filenames, stimuli_target_location)
+
+            stimuli_basenames = [os.path.basename(f) for f in stimuli_filenames]
+
+            # FixationTrains
+
+            print('Creating fixations')
+            f = zipfile.ZipFile(os.path.join(temp_dir, 'DUT-OMRON-eye-fixations.zip'))
+            f.extractall(temp_dir)
+
+            train_xs = []
+            train_ys = []
+            train_ts = []
+            train_ns = []
+            train_subjects = []
+
+            for n, basename in enumerate(tqdm(stimuli_basenames)):
+                eye_filename = os.path.join(temp_dir, 'DUT-OMRON-eye-fixations', 'mat', basename.replace('.jpg', '.mat'))
+                eye_data = loadmat(eye_filename)['s']
+                xs, ys, subject_ids = eye_data.T
+                n_fixations += len(xs) - 1  # first entry is image size
+                for subject_index in range(subject_ids.max()):
+                    subject_inds = subject_ids == subject_index + 1  # subject==0 is image size
+
+                    if not np.any(subject_inds):
+                        continue
+
+                    # since there are coordinates with value 0, we assume they are 0-indexed (although they are matlab)
+                    train_xs.append(xs[subject_inds])
+                    train_ys.append(ys[subject_inds])
+                    train_ts.append(np.arange(subject_inds.sum()))
+                    train_ns.append(n)
+                    train_subjects.append(subject_index)
+
+        fixations = FixationTrains.from_fixation_trains(train_xs, train_ys, train_ts, train_ns, train_subjects)
+
+        assert len(fixations) == n_fixations
+
+        if location:
+            stimuli.to_hdf5(os.path.join(location, 'stimuli.hdf5'))
+            fixations.to_hdf5(os.path.join(location, 'fixations.hdf5'))
+    return stimuli, fixations
