@@ -114,6 +114,10 @@ class ScanpathModel(SamplingModelMixin, object):
             out=out
         )
 
+    def conditional_log_densities(self, stimuli, fixations, verbose=False, **kwargs):
+        """ returns conditional log density predictions for each fixation """
+        return [self.conditional_log_density_for_fixation(stimuli, fixations, fixation_index) for fixation_index in tqdm(range(len(fixations)), disable=not verbose)]
+
     def log_likelihoods(self, stimuli, fixations, verbose=False):
         log_likelihoods = np.empty(len(fixations.x))
         for i in tqdm(range(len(fixations.x)), disable=not verbose):
@@ -129,6 +133,8 @@ class ScanpathModel(SamplingModelMixin, object):
         return average_values(self.log_likelihoods(stimuli, fixations, verbose=verbose), fixations, average=average)
 
     def information_gains(self, stimuli, fixations, baseline_model=None, verbose=False, average='fixation'):
+        if average != 'fixation':
+            raise NotImplementedError()
         if baseline_model is None:
             baseline_model = UniformModel()
 
@@ -175,7 +181,7 @@ class ScanpathModel(SamplingModelMixin, object):
 
         return stimuli, train_counts, lengths, stimulus_indices
 
-    def sample(self, stimuli, train_counts, lengths=1, stimulus_indices=None, rst=None):
+    def sample(self, stimuli, train_counts, lengths=1, stimulus_indices=None, rst=None, verbose=False):
         """
         Sample fixations for given stimuli
 
@@ -212,7 +218,9 @@ class ScanpathModel(SamplingModelMixin, object):
         ts = []
         ns = []
         subjects = []
-        for stimulus_index, ls in progressinfo(zip(stimulus_indices, lengths)):
+        total_count = sum(len(l) for l in lengths)
+        pbar = tqdm(total=total_count, disable=not verbose)
+        for stimulus_index, ls in zip(stimulus_indices, lengths):
             stimulus = stimuli[stimulus_index]
             for l in ls:
                 this_xs, this_ys, this_ts = self._sample_fixation_train(stimulus, l, rst=rst)
@@ -221,6 +229,7 @@ class ScanpathModel(SamplingModelMixin, object):
                 ts.append(this_ts)
                 ns.append(stimulus_index)
                 subjects.append(0)
+                pbar.update(1)
         return FixationTrains.from_fixation_trains(xs, ys, ts, ns, subjects)
 
     def _sample_fixation_train(self, stimulus, length, rst=None):
@@ -377,7 +386,7 @@ class UniformModel(Model):
 class MixtureModel(Model):
     """ A saliency model being a weighted mixture of a number of other models
     """
-    def __init__(self, models, weights=None, **kwargs):
+    def __init__(self, models, weights=None, check_norm=True, **kwargs):
         """Create a mixture model from a list of models and a list of weights
 
            :param models: list of `Model` instances
@@ -394,6 +403,7 @@ class MixtureModel(Model):
         if not len(weights) == len(models):
             raise ValueError('models and weights must have same length!')
         self.weights = weights
+        self.check_norm = check_norm
 
     def _log_density(self, stimulus):
         log_densities = []
@@ -403,7 +413,47 @@ class MixtureModel(Model):
             log_densities.append(log_density)
 
         log_density = logsumexp(log_densities, axis=0)
-        np.testing.assert_allclose(np.exp(log_density).sum(), 1.0, rtol=1e-7)
+
+        if self.check_norm:
+            np.testing.assert_allclose(np.exp(log_density).sum(), 1.0, rtol=1e-7)
+
+        if not log_density.shape == (stimulus.shape[0], stimulus.shape[1]):
+            raise ValueError('wrong density shape in mixture model! stimulus shape: ({}, {}), density shape: {}'.format(stimulus.shape[0], stimulus.shape[1], log_density.shape))
+        return log_density
+
+
+class MixtureScanpathModel(ScanpathModel):
+    """ A scanpath model being a weighted mixture of a number of other models
+    """
+    def __init__(self, models, weights=None, check_norm=True, **kwargs):
+        """Create a mixture scanpath model from a list of models and a list of weights
+
+           :param models: list of `ScanpathModel` instances
+           :param weights: list of weights for the different models. Do not have
+                           to sum up to one, they will be normalized.
+                           If `None`, will be set to a uniform mixture.
+        """
+        super(MixtureScanpathModel, self).__init__(**kwargs)
+        self.models = models
+        if weights is None:
+            weights = np.ones(len(self.models))
+        weights = np.asarray(weights, dtype=float)
+        weights /= weights.sum()
+        if not len(weights) == len(models):
+            raise ValueError('models and weights must have same length!')
+        self.weights = weights
+        self.check_norm = check_norm
+
+    def conditional_log_density(self, stimulus, x_hist, y_hist, t_hist, attributes=None, out=None):
+        log_densities = []
+        for i, model in enumerate(self.models):
+            log_density = model.conditional_log_density(stimulus, x_hist, y_hist, t_hist, attributes=attributes).copy()
+            log_density += np.log(self.weights[i])
+            log_densities.append(log_density)
+
+        log_density = logsumexp(log_densities, axis=0)
+        if self.check_norm:
+            np.testing.assert_allclose(np.exp(log_density).sum(), 1.0, rtol=1e-7)
         if not log_density.shape == (stimulus.shape[0], stimulus.shape[1]):
             raise ValueError('wrong density shape in mixture model! stimulus shape: ({}, {}), density shape: {}'.format(stimulus.shape[0], stimulus.shape[1], log_density.shape))
         return log_density
@@ -500,9 +550,10 @@ class SubjectDependentModel(DisjointUnionModel):
 
 
 class StimulusDependentModel(Model):
-    def __init__(self, stimuli_models, check_stimuli=True, **kwargs):
+    def __init__(self, stimuli_models, check_stimuli=True, fallback_model=None, **kwargs):
         super(StimulusDependentModel, self).__init__(**kwargs)
         self.stimuli_models = stimuli_models
+        self.fallback_model = fallback_model
         if check_stimuli:
             self.check_stimuli()
 
@@ -517,13 +568,17 @@ class StimulusDependentModel(Model):
             if stimulus_hash in stimuli.stimulus_ids:
                 return model.log_density(stimulus)
         else:
-            raise ValueError('stimulus not provided by these models')
+            if self.fallback_model is not None:
+                return self.fallback_model.log_density(stimulus)
+            else:
+                raise ValueError('stimulus not provided by these models')
 
 
 class StimulusDependentScanpathModel(ScanpathModel):
-    def __init__(self, stimuli_models, check_stimuli=True, **kwargs):
+    def __init__(self, stimuli_models, check_stimuli=True, fallback_model=None, **kwargs):
         super(StimulusDependentScanpathModel, self).__init__(**kwargs)
         self.stimuli_models = stimuli_models
+        self.fallback_model = fallback_model
         if check_stimuli:
             self.check_stimuli()
 
@@ -538,7 +593,10 @@ class StimulusDependentScanpathModel(ScanpathModel):
             if stimulus_hash in stimuli.stimulus_ids:
                 return model.conditional_log_density(stimulus, x_hist, y_hist, t_hist, attributes=attributes, out=out)
         else:
-            raise ValueError('stimulus not provided by these models')
+            if self.fallback_model is not None:
+                return self.fallback_model.conditional_log_density(stimulus, x_hist, y_hist, t_hist, attributes=attributes, out=out)
+            else:
+                raise ValueError('stimulus not provided by these models')
 
 
 class FixationIndexDependentModel(ScanpathModel):
@@ -569,9 +627,27 @@ class ShuffledAUCSaliencyMapModel(SaliencyMapModel):
         return self.probabilistic_model.log_density(stimulus) - self.baseline_model.log_density(stimulus)
 
 
+def average_predictions(predictions, library):
+    predictions = np.array(predictions) - np.log(len(predictions))
+
+    if library == 'tensorflow':
+        from .tf_utils import tf_logsumexp
+        prediction = tf_logsumexp(predictions, axis=0)
+    elif library == 'torch':
+        import torch
+        device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
+        prediction = torch.logsumexp(torch.Tensor(predictions).to(device), dim=0).detach().cpu().numpy()
+    elif library == 'numpy':
+        prediction = logsumexp(predictions, axis=0)
+    else:
+        raise ValueError(library)
+
+    return prediction
+
+
 class ShuffledBaselineModel(Model):
     """Predicts a mixture of all predictions for other images.
-    
+
     This model will usually be used as baseline model for computing sAUC saliency maps.
 
     use the library parameter to define whether the logsumexp should be computed
@@ -624,20 +700,67 @@ class ShuffledBaselineModel(Model):
             other_prediction = self.resized_predictions_cache[k]
             predictions.append(other_prediction)
 
-        predictions = np.array(predictions) - np.log(len(predictions))
+        prediction = average_predictions(predictions, self.library)
 
-        if self.library == 'tensorflow':
-            from .tf_utils import tf_logsumexp
-            prediction = tf_logsumexp(predictions, axis=0)
-        elif self.library == 'torch':
-            import torch
-            device = torch.device("cuda" if torch.cuda.is_available() else "cpu")
-            prediction = torch.logsumexp(torch.Tensor(predictions).to(device), dim=0).detach().cpu().numpy()
-        elif self.library == 'numpy':
-            prediction = logsumexp(predictions, axis=0)
-        else:
-            raise ValueError(self.library)
+        prediction = self._resize_prediction(prediction, target_shape)
 
+        return prediction
+
+
+class ShuffledSimpleBaselineModel(Model):
+    """Predicts a mixture of all predictions for all images.
+
+    This model will usually be used as baseline model for computing sAUC saliency maps
+    when the ShuffledBaselineModel is not feasible.
+
+    use the library parameter to define whether the logsumexp should be computed
+    with torch (default), tensorflow or numpy.
+    """
+    def __init__(self, parent_model, stimuli,
+                 compute_size=(500, 500),
+                 library='torch',
+                 **kwargs):
+        super(ShuffledSimpleBaselineModel, self).__init__(**kwargs)
+        self.parent_model = parent_model
+        self.stimuli = stimuli
+        self.compute_size = compute_size
+        self.prediction = None
+        if library not in ['torch', 'tensorflow', 'numpy']:
+            raise ValueError(library)
+        self.library = library
+
+    def get_average_prediction(self, verbose=False):
+        if self.prediction is not None:
+            return self.prediction
+
+        predictions = []
+        for stimulus in tqdm(self.stimuli, disable=not verbose):
+            predictions.append(
+                self._resize_prediction(self.parent_model.log_density(stimulus), self.compute_size)
+            )
+
+        prediction = average_predictions(predictions, self.library)
+
+        self.prediction = prediction
+        return self.prediction
+
+    def _resize_prediction(self, prediction, target_shape):
+        if prediction.shape != target_shape:
+            x_factor = target_shape[1] / prediction.shape[1]
+            y_factor = target_shape[0] / prediction.shape[0]
+
+            prediction = zoom(prediction, [y_factor, x_factor], order=1, mode='nearest')
+
+            prediction -= logsumexp(prediction)
+
+            assert prediction.shape == target_shape
+
+        return prediction
+
+    def _log_density(self, stimulus):
+        prediction = self.get_average_prediction()
+
+        target_shape = (stimulus.shape[0], stimulus.shape[1])
         prediction = self._resize_prediction(prediction, target_shape)
 
         return prediction
@@ -669,5 +792,99 @@ class SaliencyMapNormalizingModel(Model):
         return np.log(smap)
 
 
+class FixedStimulusSizeModel(Model):
+    """ model which scales images to have a fixed size before handing them to anothet model"""
+    def __init__(self, size, parent_model, verbose=False, **kwargs):
+        super(FixedStimulusSizeModel, self).__init__(**kwargs)
+
+        self.size = size
+        self.parent_model = parent_model
+        self.verbose = verbose
+
+    def _log_density(self, stimulus):
+        stimulus = self.ensure_color(stimulus)
+
+        stimulus_size = stimulus.shape[0], stimulus.shape[1]
+        max_size = max(stimulus_size)
+        factor = self.size / max_size
+
+        if factor != 1.0:
+            if self.verbose:
+                print("Resizing with factor", factor)
+            stimulus_for_parent_model = zoom(stimulus, [factor, factor, 1.0], order=1, mode='nearest')
+        else:
+            stimulus_for_parent_model = stimulus
+
+        log_density = self.parent_model.log_density(stimulus_for_parent_model)
+
+        factor_y = stimulus.shape[0] / log_density.shape[0]
+        factor_x = stimulus.shape[1] / log_density.shape[1]
+
+        if factor_y != 1.0 or factor_x != 1.0:
+            if self.verbose:
+                print("Wrong shape, resizing log densities", stimulus.shape, log_density.shape)
+            log_density = zoom(log_density, [factor_y, factor_x], order=1, mode='nearest')
+            log_density -= logsumexp(log_density)
+
+        assert log_density.shape[0] == stimulus.shape[0]
+        assert log_density.shape[1] == stimulus.shape[1]
+
+        return log_density
+
+    def ensure_color(self, image):
+        if image.ndim == 2:
+            return np.dstack((image, image, image))
+        return image
+
+
+class DVAAwareModel(Model):
+    """ A model which adapts another model to a new image resolution by rescaling images before computing predictions
+
+    - dva: expected image resolution in pixel per dva for this model
+    - parent_model_dva: image resolution expected by parent_model
+    """
+    def __init__(self, dva, parent_model, parent_model_dva, verbose=False, **kwargs):
+
+        super(DVAAwareModel, self).__init__(**kwargs)
+
+        self.dva = dva
+        self.parent_model = parent_model
+        self.parent_model_dva = parent_model_dva
+        self.verbose = verbose
+
+        self.factor = self.parent_model_dva / self.dva
+
+    def _log_density(self, stimulus):
+        stimulus = self.ensure_color(stimulus)
+
+        if self.factor != 1.0:
+            if self.verbose:
+                print("Resizing with factor", self.factor)
+            stimulus_for_parent_model = zoom(stimulus, [self.factor, self.factor, 1.0], order=1, mode='nearest')
+        else:
+            stimulus_for_parent_model = stimulus
+
+        log_density = self.parent_model.log_density(stimulus_for_parent_model)
+
+        factor_y = stimulus.shape[0] / log_density.shape[0]
+        factor_x = stimulus.shape[1] / log_density.shape[1]
+
+        if factor_y != 1.0 or factor_x != 1.0:
+            if self.verbose:
+                print("Wrong shape, resizing log densities", stimulus.shape, log_density.shape)
+            log_density = zoom(log_density, [factor_y, factor_x], order=1, mode='nearest')
+            log_density -= logsumexp(log_density)
+
+        assert log_density.shape[0] == stimulus.shape[0]
+        assert log_density.shape[1] == stimulus.shape[1]
+
+        return log_density
+
+    def ensure_color(self, image):
+        if image.ndim == 2:
+            return np.dstack((image, image, image))
+        return image
+
+
 GeneralModel = deprecated_class(deprecated_in='0.2.16', removed_in='1.0.0', details="Use ScanpathModel instead")(ScanpathModel)
 StimulusDependentGeneralModel = deprecated_class(deprecated_in='0.2.16', removed_in='1.0.0', details="Use StimulusDependentScanpathModel instead")(StimulusDependentScanpathModel)
