@@ -13,7 +13,7 @@ from tqdm import tqdm
 from boltons.cacheutils import cached, LRU
 
 from .roc import general_roc, general_rocs_per_positive
-from .numba_utils import fill_fixation_map
+from .numba_utils import fill_fixation_map, auc_for_one_positive
 
 from .utils import TemporaryDirectory, run_matlab_cmd, Cache, average_values, deprecated_class, remove_trailing_nans
 from .datasets import Stimulus, Fixations
@@ -110,6 +110,10 @@ class ScanpathSaliencyMapModel(object):
             out=out
         )
 
+    def conditional_saliency_maps(self, stimuli, fixations, verbose=False, **kwargs):
+        """ returns conditional log density predictions for each fixation """
+        return [self.conditional_saliency_map_for_fixation(stimuli, fixations, fixation_index) for fixation_index in tqdm(range(len(fixations)), disable=not verbose)]
+
     def AUCs(self, stimuli, fixations, nonfixations='uniform', verbose=False):
         """
         Calulate AUC scores for fixations
@@ -142,7 +146,7 @@ class ScanpathSaliencyMapModel(object):
         if isinstance(nonfixations, Fixations):
             nonfix_xs = []
             nonfix_ys = []
-            for n in range(fixations.n.max()+1):
+            for n in range(fixations.n.max() + 1):
                 inds = nonfixations.n == n
                 nonfix_xs.append(nonfixations.x_int[inds].copy())
                 nonfix_ys.append(nonfixations.y_int[inds].copy())
@@ -152,7 +156,7 @@ class ScanpathSaliencyMapModel(object):
 
         for i in tqdm(range(len(fixations.x)), total=len(fixations.x), disable=not verbose):
             out = self.conditional_saliency_map_for_fixation(stimuli, fixations, i, out=out)
-            positives = np.asarray([out[fixations.y_int[i], fixations.x_int[i]]])
+            positive = out[fixations.y_int[i], fixations.x_int[i]]
             if nonfixations == 'uniform':
                 negatives = out.flatten()
             elif nonfixations == 'unfixated':
@@ -169,10 +173,7 @@ class ScanpathSaliencyMapModel(object):
             else:
                 raise ValueError("Don't know how to handle nonfixations {}".format(nonfixations))
 
-            positives = positives.astype(float)
-            negatives = negatives.astype(float)
-
-            this_roc, _, _ = general_roc(positives, negatives)
+            this_roc = auc_for_one_positive(positive, negatives)
             rocs.setdefault(fixations.n[i], []).append(this_roc)
             rocs_per_fixation.append(this_roc)
         return np.asarray(rocs_per_fixation)
@@ -288,7 +289,7 @@ class SaliencyMapModel(ScanpathSaliencyMapModel):
     def conditional_saliency_map(self, stimulus, *args, **kwargs):
         return self.saliency_map(stimulus)
 
-    def AUCs(self, stimuli, fixations, nonfixations='uniform', verbose=False):
+    def AUCs(self, stimuli, fixations, nonfixations='uniform', thresholds='all', verbose=False):
         """
         Calulate AUC scores for fixations
 
@@ -314,6 +315,13 @@ class SaliencyMapModel(ScanpathSaliencyMapModel):
 
         nonfix_ys = None
         nonfix_xs = None
+        
+        if thresholds == 'all':
+            judd = 0
+        elif thresholds == 'fixations':
+            judd = 1
+        else:
+            raise ValueError("Unknown value of `thresholds`: {}".format(thresholds))
 
         if isinstance(nonfixations, Fixations):
             nonfix_xs = []
@@ -326,7 +334,7 @@ class SaliencyMapModel(ScanpathSaliencyMapModel):
         if nonfixations == 'shuffled':
             nonfixations = FullShuffledNonfixationProvider(stimuli, fixations)
 
-        for n in tqdm(range(len(stimuli)), total=len(stimuli), disable = not verbose):
+        for n in tqdm(range(len(stimuli)), total=len(stimuli), disable=not verbose):
             inds = fixations.n == n
             if not inds.sum():
                 continue
@@ -350,7 +358,7 @@ class SaliencyMapModel(ScanpathSaliencyMapModel):
             positives = positives.astype(float)
             negatives = negatives.astype(float)
 
-            rocs = general_rocs_per_positive(positives, negatives)
+            rocs = general_rocs_per_positive(positives, negatives, judd=judd)
             rocs_per_fixation[inds] = rocs
 
         return rocs_per_fixation
@@ -397,7 +405,7 @@ class SaliencyMapModel(ScanpathSaliencyMapModel):
         if isinstance(nonfixations, Fixations):
             nonfix_xs = []
             nonfix_ys = []
-            for n in range(fixations.n.max()+1):
+            for n in range(fixations.n.max() + 1):
                 inds = nonfixations.n == n
                 nonfix_xs.append(nonfixations.x_int[inds].copy())
                 nonfix_ys.append(nonfixations.y_int[inds].copy())
@@ -481,7 +489,7 @@ class SaliencyMapModel(ScanpathSaliencyMapModel):
         else:
             raise ValueError(average)
 
-    def AUC_Judd(self, stimuli, fixations, jitter=True, noise_size=1.0/10000000, random_seed=42, verbose=False):
+    def AUCs_Judd(self, stimuli, fixations, jitter=True, noise_size=1.0/10000000, random_seed=42, verbose=False):
         if jitter:
             model = RandomNoiseSaliencyMapModel(
                 self,
@@ -490,14 +498,19 @@ class SaliencyMapModel(ScanpathSaliencyMapModel):
             )
         else:
             model = self
-        return model.AUC(
+
+        return model.AUCs(
             stimuli,
             fixations,
-            average='image',
             nonfixations='unfixated',
             thresholds='fixations',
             verbose=verbose
         )
+    
+    def AUC_Judd(self, stimuli, fixations, jitter=True, noise_size=1.0/10000000, random_seed=42, verbose=False):
+        aucs = self.AUCs_Judd(stimuli, fixations, jitter=jitter, noise_size=noise_size, random_seed=random_seed, verbose=verbose)
+        return average_values(aucs, fixations, average='image')
+
 
     def fixation_based_KL_divergence(self, stimuli, fixations, nonfixations='shuffled', bins=10, eps=1e-20):
         """
